#include <intrinsic.h>
#include "iotms470r1b1m.h"
#include "tms470r1b1m_bit_definitions.h"
#include "std_het.h"
#include "audio_playback_H.h"
#include "speex\speex.h"
#include "VoiceMessage1.h"	        // Include the compressed voice data
#include "VoiceMessage2.h"	        // Include the compressed voice data
#include "VoiceMessage3.h"	        // Include the compressed voice data

#if NBYTES_DEC1 > NBYTES_DEC2
#define NBYTES_DEC_TEMP NBYTES_DEC1
#else
#define NBYTES_DEC_TEMP NBYTES_DEC2
#endif
#if NBYTES_DEC3 > NBYTES_DEC_TEMP
#define NBYTES_DEC NBYTES_DEC3
#else
#define NBYTES_DEC NBYTES_DEC_TEMP
#endif

// Declare the Speex data structures
SpeexBits bits;						// Speex bit packing structure
void *dec_state;					// Decoder state

// Other global variables
const char *data_enc_ptr;				// pointer to encoded data (PCM)
char StartSpeex = 0;
char data_dec[2][NBYTES_DEC];	   // decoded frame (PCM), double buffering
char *data_dec_ptr;                // pointer to decoded frame (PCM)
char dec_buffer;                   // buffer to decode into
char play_buffer;                  // buffer to playback from
unsigned int play_index;           // Current sample to play
unsigned int play_count;           // Amount samples to play

volatile int NFrames;
volatile int NBytesDec;
volatile int NBytesEnc;

spx_int16_t data_dec_frame[NBYTES_DEC];		// decoded frame
spx_int16_t *data_dec_frame_ptr;			// pointer to decoded frame

// HET Program load address
__no_init volatile HETPROGRAM0_UN e_HETPROGRAM0_UN @ 0x00800000;

// Function prototypes
void MemCopy32(unsigned long *dst, unsigned long *src, int bytes);
void SpeexMain(void);
void SpeexInit(void);
//-------------------------------------------------------------------------
void main(void)
{
  PCR = CLKDIV_2;                               // ICLK=SYSCLK/2
                                                // (to not violate max. ICLK)
  PCR |= PENABLE;				// Enable all peripheral clocks
                                                // AFTER setting CLKDIV

  HETDIR  |= 0xFFFFFFFFL;			// Setup all HETx pins as outputs
  HETDOUT  = 0xFFFFFFFFL;			// Set all HETx pins (all LED's off)

  GIOENA1 = A4 + A5 + A6;                       // Enable int for buttons B1, B2, B3
//  GIOENA1 |= A7;                                // Enable int for button B4
//  GIOENA1 |= A2;                                // Enable int for button BUT
  GIOPOL1 = 0x00;                               // Rising polarity
  GIOPRY1 = 0x00;                               // Set to low priority (GIOB)
  GIOFLG1 = 0x00;                               // Clear GIO int flag register

  REQMASK = (1 << CIM_GIOB) + (1 << CIM_HET1);  // Enable GIOB and HET1 ints
//  FIRQPR = (1 << CIM_GIOB);                     // GPIO Interrupt is FIQ Type
//  FIRQPR = (1 << CIM_HET1);                     // HET Interrupt is FIQ Type
  
  // Copy HET instructions to HET RAM
  MemCopy32((void *) &e_HETPROGRAM0_UN,
            (void *) HET_INIT0_PST,
            sizeof(HET_INIT0_PST));

  HETGCR = CLK_MASTER + IGNORE_SUSPEND;         // HET Master Mode,
                                                // Ignore SW BP
  HETPRY = 0x010;                               // is high priority
  HETPFR = LRPRES_FACTOR_32 + HRPRES_FACTOR_10;

  SpeexInit();  //This should only be called once!

  __enable_interrupt();                         // Enable Interrupts

  while(1)
  {
    if(StartSpeex)
      SpeexMain();
  }
}

//------------------------------------------------------------------------------
// Main Function to interface the CPU to the HET and pass on the Speex code.
//------------------------------------------------------------------------------
void SpeexMain(void)
{
  // Initialize the data array pointers
  dec_buffer = 1;                         // Decode to buffer 0 now
//  data_enc_ptr = data_enc;                // Assign pointer
  data_dec_ptr = data_dec[dec_buffer];	 	// Assign pointer
  data_dec_frame_ptr = data_dec_frame;		// Assign pointer

  // Init playback of buffer 0
  play_index = 0;                   // First sample to play
  dec_buffer = 0;                   // Decode buffer 0 first
  play_buffer = 0;                  // Playback buffer 0 first
  play_count = NBYTES_DEC;          // Bytes to playback

  HETGCR |= ON;                     // Start HET program (starts playback of buffer 0)

  for(int i=0; i<NFrames-1; i++)    // Loop through frames
  {
    speex_bits_read_from(&bits, (char *)data_enc_ptr, NBytesEnc);  // Point to the next data frame
    speex_decode_int(dec_state, &bits, data_dec_frame_ptr);         // Decompress the frame

    for(int j=0; j<NBYTES_DEC; j++) // Transfer the output frame to the 8-bit PCM output buffer
    {
      *data_dec_ptr++ = (char)(((spx_int16_t)(*data_dec_frame_ptr++) >> 8) + 0x80);
    }

    while (dec_buffer != play_buffer);  // Wait for last buffer to be played by HET
    dec_buffer = dec_buffer ^ 1;        // switch decode buffer: if 0 then 1, if 1 then 0
    data_enc_ptr += NBytesEnc;		// Advance the encoded data pointer
    data_dec_ptr = data_dec[dec_buffer];// Re-init the decoded data pointer (PCM)
    data_dec_frame_ptr = data_dec_frame;// Re-init the decompressed data pointer (float)
  }
  while (dec_buffer != play_buffer);    // Wait for last buffer to be played
  HETGCR &= ~ON;                        // Stop HET program
  StartSpeex = 0;                       // Reset playback flag
} // end of SpeexMain()

//------------------------------------------------------------------------------
// Intialization Function required for the Speex code.
// If this function is called everytime then the Vocoder did not work after the
// third time of these functions being called.
//------------------------------------------------------------------------------
void SpeexInit(void)
{
  // Initialize the Speex decoder
  int tmp  = 0;
  speex_bits_init(&bits);			                  // Init the bit packing structure
  dec_state = speex_decoder_init(&speex_nb_mode);                 // Init the decoder state
  speex_decoder_ctl(dec_state, SPEEX_SET_ENH, &tmp);  // Turn off perceptual enhancement
}

//------------------------------------------------------------------------------
// Help function to load the HET program into the HET RAM using 32-bit access.
//------------------------------------------------------------------------------
void MemCopy32(unsigned long *dst, unsigned long *src, int bytes)
{
  for (int i = 0; i < (bytes + 3) / 4; i++)
    *dst++ = *src++;
}
//------------------------------------------------------------------------------
// This ISR is called with a frequency of 23.4375kHz and is used to feed
// the HET PWM output by directly writing to the data field of the MOV32
// instruction on every 3rd ISR entry. This functionality could be moved
// into the HET program for CPU load reduction.
//
// HET4 will get toggled with 23.4375kHz / 3 / 2 = 3.9kHz, indicating
// a playback sample rate of 7.8kHz.
//------------------------------------------------------------------------------
void HET1_Instr0_handler()
{
  HETDOUT ^= 0x00000010;                        // Toggle HET4 for update
                                                // frequency verification
                                                // (HET4 will show fUPD / 2)

  HET_load_new_pwm_data_01_0.memory.data_word = data_dec[play_buffer][play_index];

  if (++play_index == play_count)               // Buffer finished?
  {
    play_index = 0;                             // Reset playback index
    play_buffer = play_buffer ^ 1;              // Switch playback buffer: if 0 then 1, if 1 then 0
  }
}
//------------------------------------------------------------------------------
// HET Interrupt Level 1 Handler
//------------------------------------------------------------------------------
void HET1_irq_handler()
{
  switch ((HETOFF1 & 0xff) - 1)
  {
    case 4 :                    // Int on instruction 0
      HET1_Instr0_handler();
      break;
  }
}
//------------------------------------------------------------------------------
// GIOB Interrupt Handler
//------------------------------------------------------------------------------
void GIOB_irq_handler(void)
{
  if (GIOFLG1 & A4)   // Was B1 pressed?
  {
    StartSpeex = 1;   // Set playback flag
    GIOFLG1 = A4;     // Clear button int flag
    NFrames = NFRAMES1;
    NBytesDec = NBYTES_DEC1;
    NBytesEnc = NBYTES_ENC1;
    data_enc_ptr = data_enc1; // Assign pointer to Speex Data Array
  }
  else
  {
    if (GIOFLG1 & A5)   // Was B2 pressed?
    {
      StartSpeex = 1;   // Set playback flag
      GIOFLG1 = A5;     // Clear button int flag
      NFrames = NFRAMES2;
      NBytesDec = NBYTES_DEC2;
      NBytesEnc = NBYTES_ENC2;
      data_enc_ptr = data_enc2; // Assign pointer to Speex Data Array
    }
    else
    {
      if (GIOFLG1 & A6)   // Was B3 pressed?
      {
        StartSpeex = 1;   // Set playback flag
        GIOFLG1 = A6;     // Clear button int flag
        NFrames = NFRAMES3;
        NBytesDec = NBYTES_DEC3;
        NBytesEnc = NBYTES_ENC3;
        data_enc_ptr = data_enc3; // Assign pointer to Speex Data Array
      }
    }
  }
}
//------------------------------------------------------------------------------
// TMS470R1B1M Standard Interrupt Handler
//------------------------------------------------------------------------------
__irq __arm void irq_handler(void)
{
  switch ((IRQIVEC & 0xff) - 1)
  {
    case CIM_HET1 :
      HET1_irq_handler();
      break;
    case CIM_GIOB :       // GIO interrupt B
      GIOB_irq_handler();
      break;
  }
}

//------------------------------------------------------------------------------
// TMS470R1B1M Fast Interrupt Handler
// The FIQ is not working for the Vocoder HET1
//------------------------------------------------------------------------------
__fiq __arm void fiq_handler(void)
{
  switch ((FIQIVEC & 0xff) - 1)
  {
    case CIM_HET1 :
      HET1_irq_handler();
      break;
    case CIM_GIOB :       // GIO interrupt B
      GIOB_irq_handler();
      break;
  }
}

