#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#ifndef NULL
#define NULL 0
#endif

#include "SpeexFunctions.h"

/** Maximum size of the bit-stream (for fixed-size allocation) */
//#define MAX_BYTES_PER_FRAME (2000/BYTES_PER_CHAR)
//ALTER: changed 2000 to 160
#define MAX_BYTES_PER_FRAME (160/BYTES_PER_CHAR)

void speex_bits_init(SpeexBits *bits)
{
   bits->chars = (char*)speex_alloc(MAX_BYTES_PER_FRAME);
   bits->buf_size = MAX_BYTES_PER_FRAME;

   bits->chars[0]=0;
   bits->nbBits=0;
   bits->charPtr=0;
   bits->bitPtr=0;
   bits->owner=1;
   bits->overflow=0;
}

void *speex_alloc (int size)
{
   return calloc(size,1);
}

void *speex_realloc (void *ptr, int size)
{
   return realloc(ptr, size);
}


void *speex_decoder_init(const SpeexMode *mode)
{
   return mode->dec_init(mode);
}

int speex_decoder_ctl(void *state, int request, void *ptr)
{
   return (*((SpeexMode**)state))->dec_ctl(state, request, ptr);
}


void speex_bits_read_from(SpeexBits *bits, char *chars, int len)
{
   int i;
   if (len > bits->buf_size)
   {
      //speex_warning_int("Packet if larger than allocated buffer: ", len);
      if (bits->owner)
      {
         char *tmp = (char*)speex_realloc(bits->chars, len);
         if (tmp)
         {
            bits->buf_size=len;
            bits->chars=tmp;
         } else {
            len=bits->buf_size;
            //speex_warning("Could not resize input buffer: truncating input");
         }
      } else {
         //speex_warning("Do not own input buffer: truncating input");
         len=bits->buf_size;
      }
   }
   for (i=0;i<len;i++)
      bits->chars[i]=chars[i];
   bits->nbBits=len<<3;
   bits->charPtr=0;
   bits->bitPtr=0;
   bits->overflow=0;
}


int speex_decode_int(void *state, SpeexBits *bits, spx_int16_t *out)
{
   SpeexMode *mode = *(SpeexMode**)state;
   return (mode)->dec(state, bits, out);
}

